Emotional = LibStub("AceAddon-3.0"):NewAddon("Emotional", "AceConsole-3.0", "AceEvent-3.0")

local defaults = {
    char = {
		configured = false,
		enableInParty = true,
		enableInRaid = true,
		enableInPvP = true,
		enableInLFR = false,
		enableDebug = false,
		emotes = {
				applaud = false,
				cackle = false,
				cheer = false,
				gloat = false,
				golfclap = false,
				guffaw = false,
				moo = false,
				mourn = false,
				rolleyes = false,
				sigh = false,
				sniff = false,
				train = false,
				violin = false,
		},
		available = {
				}
		}
	}

local options = {
    name = "Emotional",
    handler = Emotional,
    type = 'group',
    args = {
		version = {
			order = 0,
			type = "description",
			name = "v4.0",
			width = "full",
		},
		desc = {
			order = 1,
			type = "description",
			name = "Death Makes Me Emotional",
			width = "full",
		},
		emote = {
			order = 20,
			type = "multiselect",
			name = "Emotion (Randomly from selected)",
			desc = "List of Available Emotions",
			get = "IsEnableEmotes",
			set = "ToggleEnableEmotes",
			width = "full",
			values = {
				["applaud"] = "You applaud at <target>. Bravo!",
				["cackle"] = "You cackle maniacally at <target>.",
				["cheer"] = "You cheer at <target>.",
				["gloat"] = "You gloat over <target>'s misfortune.",
				["golfclap"] = "You clap for <target>, clearly unimpressed.",
				["guffaw"] = "You take one look at <target> and let out a guffaw!",
				["moo"] = "You moo at <target>. Mooooooooooo.",
				["mourn"] = "In quiet contemplation, you mourn the death of <target>.",
				["rolleyes"] = "You roll your eyes at <target>.",
				["sigh"] = "You sigh a <target>.",
				["sniff"] = "You sniff <target>.",
				["train"] = "Choo Choo!",
				["violin"] = "You play the world's smallest violin for <target>.",
			}
		},
		enableInParty = {
			order = 16,
			type = "toggle",
			name = "Emotional in Parties",
			desc = "Enable/Disable in 5man Groups",
			get = "IsEnableInParty",
			set = "ToggleEnableInParty",
			width = "full"
		},
		enableInRaid = {
			order = 17,
			type = "toggle",
			name = "Emotional in Raids",
			desc = "Enable/Disable in Raid Groups",
			get = "IsEnableInRaid",
			set = "ToggleEnableInRaid",
			width = "full"
		},
		enableInPvP = {
			order = 15,
			type = "toggle",
			name = "Emotional in PvP",
			desc = "Enable/Disable in PvP Groups",
			get = "IsEnableInPvP",
			set = "ToggleEnableInPvP",
			width = "full"
		},
		enableInLFR = {
			order = 10,
			type = "toggle",
			name = "Emotional in LFR",
			desc = "Enable/Disable in LFR Groups",
			get = "IsEnableInLFR",
			set = "ToggleEnableInLFR",	
		},
    },
}

function Emotional:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("EmotionalDB", defaults)
    LibStub("AceConfig-3.0"):RegisterOptionsTable("Emotional", options)
	self.EmotionalOptFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("Emotional", "Emotional")
end

function Emotional:DisplayStatus()
	print("|cff0078c1Emotional|r"," Version: |cff0078c1",options.args.version.name,"|r")
	print(options.args.desc.name)
	if self.db.char.enableInLFR then
		print("|cff58b058LFR|r")
	else
		print("|cfff05032LFR|r")
	end
	if self.db.char.enableInPvP then
		print("|cff58b058PvP|r")
	else
		print("|cfff05032PvP|r")
	end
	if self.db.char.enableInParty then
		print("|cff58b058Party|r")
	else
		print("|cfff05032Party|r")
	end
	if self.db.char.enableInRaid then
		print("|cff58b058Raid|r")
	else
		print("|cfff05032Raid|r")
	end
	self:DisplayEmotes()
end

function Emotional:DisplayInfo()
	print("Welcome to |cff0078c1Emotional|r"," Version: |cff0078c1",options.args.version.name,"|r Loaded")
	print(options.args.desc.name)
	print("Type |cff0078c1/emo|r to access GUI menu")
end

function Emotional:DisplayMenu()
	print("|cff0078c1Emotional|r"," Version: |cff0078c1",options.args.version.name,"|r")
	print(options.args.desc.name)
	print("/emo |cff0078c1-- GUI Menu|r")
	print("/emo lfr |cff0078c1-- Enable/Disable in LFR|r")	
	print("/emo pvp |cff0078c1-- Enable/Disable in PvP|r")
	print("/emo party |cff0078c1-- Enable/Disable in Party|r")
	print("/emo raid |cff0078c1-- Enable/Disable in Raid|r")
	print("/emo help |cff0078c1-- Display Help Menu|r")
	print("/emo status |cff0078c1-- Display Status|r")
end

function Emotional:ChatCommand(input)
	if input == "debug" then
		if (self.db.char.enableDebug) then
			self.db.char.enableDebug = false
			print("Debug set to |cfff05032Disabled|r")
		else
			self.db.char.enableDebug = true
			print("Debug set to |cff58b058Enabled|r")
		end
	elseif input == "raid" then
		if (self.db.char.enableInRaid) then
			self.db.char.enableInRaid = false
			print("Raid set to |cfff05032Disabled|r")
		else
			self.db.char.enableInRaid = true
			print("Raid set to |cff58b058Enabled|r")
		end
	elseif input == "party" then
		if (self.db.char.enableInParty) then
			self.db.char.enableInParty = false
			print("Party set to |cfff05032Disabled|r")
		else
			self.db.char.enableInParty = true
			print("Party set to |cff58b058Enabled|r")
		end
	elseif input == "pvp" then
		if (self.db.char.enableInPvP) then
			self.db.char.enableInPvP = false
			print("PvP set to |cfff05032Disabled|r")
		else
			self.db.char.enableInPvP = true
			print("PvP set to |cff58b058Enabled|r")
		end
	elseif input == "lfr" then
		if (self.db.char.enableInLFR) then
			self.db.char.enableInLFR = false
			print("LFR set to |cfff05032Disabled|r")
		else
			self.db.char.enableInLFR = true
			print("LFR set to |cff58b058Enabled|r")
		end
	elseif input == "test" then
			self:RunEmote(self:ChooseEmote(),"none")
	elseif input == "status" then
			self:DisplayStatus()
	elseif input == "help" then
		self:DisplayMenu()
	elseif input == "reset" then
		self:Reset()
	elseif input == "" then
		InterfaceOptionsFrame_OpenToCategory(self.EmotionalOptFrame)
		InterfaceOptionsFrame_OpenToCategory(self.EmotionalOptFrame)
	else		
		self:RunEmote("rude","none")
		print("Try /emo help ")
	end
end

function Emotional:Reset()
	print("Emotional Reset to Default")
	self.db.char.enableInParty = true
	self.db.char.enableInRaid = true
	self.db.char.enableInPvP = true
	self.db.char.enableInLFR = false
	self.db.char.enableDebug = false
	self.db.char.configured = false
end

function Emotional:DisplayEmotes()
	local available
	available = ""
	for i=table.getn(self.db.char.available),1,-1 do
		available = self.db.char.available[i] .. " " .. available
	end
	if (available == "") then
		self:NoEmotionSelected()
	else
		print("Emotions:",available)
	end
end

function Emotional:NoEmotionSelected()
		if math.random (5) == 5 then
			self:RunEmote("rolleyes","none")
		end
		print("Emotional is |cfff05032NOT|r configured")
end

function Emotional:OnEnable()
	if (UnitRace("player") ~= "Tauren") then
		options.args.emote.values["moo"] = nil
	end
	if (UnitRace("player") ~= "Worgen") then
		options.args.emote.values["sniff"] = nil
	end
	self:DisplayInfo()
	self:RegisterChatCommand("emo", "ChatCommand")
	self:RegisterChatCommand("emotional", "ChatCommand")
	self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED","Action")
end

function Emotional:OnDisable()
	self:UnregisterChatCommand("emo", "ChatCommand")
	self:UnregisterChatCommand("emotional", "ChatCommand")
	self:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
end

function Emotional:Action(...)
	if not self.db.char.configured then
		InterfaceOptionsFrame_OpenToCategory(self.EmotionalOptFrame)
		InterfaceOptionsFrame_OpenToCategory(self.EmotionalOptFrame)
		self.db.char.configured = true
	end
	if (((UnitIsPVP("player") and self.db.char.enableInPvP) or not (UnitIsPVP("player"))) and 
	((self:InLFR() and self.db.char.enableInLFR) or not (self:InLFR())) and 
	(self:InRaid() and self.db.char.enableInRaid) or
	(self:InParty() and self.db.char.enableInParty)) then
		local arg0, arg1, event, arg3, arg4, caster, arg6, arg7, arg8, target, arg10, arg11, arg12, arg13 = select(1, ...);
		if (event == "UNIT_DIED") and (UnitIsDeadOrGhost(target)) and (UnitIsPlayer(target))then
			self:RunEmote(self:ChooseEmote(),target)
			if (self.db.char.enableDebug) then
				print(target, "has died.")
			end
		end
	end
end

function Emotional:InLFR()
	local arg0, arg1, difficulty, arg3, arg4, arg5, arg6, arg7, arg8 = GetInstanceInfo()
	if (difficulty == 7 or difficulty == 17) then
		return true
	else
		return false
	end
end

function Emotional:InParty()
	local arg0, type, arg2, arg3, arg4, arg5, arg6, arg7, arg8 = GetInstanceInfo()
	if ((type == "party" or type == "scenario") or ((GetNumGroupMembers() < 6) and (GetNumGroupMembers() > 0))) then
		return true
	else
		return false
	end
end

function Emotional:InRaid()
	local arg0, type, arg2, arg3, arg4, arg5, arg6, arg7, arg8 = GetInstanceInfo()
	if ((type == "raid") or (GetNumGroupMembers() > 5)) then
		return true
	else
		return false
	end
end

function Emotional:RunTest()
	self:RunEmote(self:ChooseEmote(),"none")
end

function Emotional:IsEnableEmotes(info, key)
	return self.db.char.emotes[key]
end

function Emotional:ToggleEnableEmotes(info, key, newValue)
	if newValue then
		self:RunEmote(key,"none")
		table.insert(self.db.char.available, key)
	else
		for i=1,table.getn(self.db.char.available),1 do
			if self.db.char.available[i] == key then
				table.remove(self.db.char.available, i)
				break
			end
		end
	end
	self.db.char.emotes[key] = newValue
end

function Emotional:RunEmote(emotion, target)
	if (emotion == "") then
		self:NoEmotionSelected()
	else
		DoEmote(emotion,target)
	end
	if (self.db.char.enableDebug) then
		if target == "none" then
			print(emotion)
		else
			print(emotion," at ",target)
		end
	end
end

function Emotional:ChooseEmote()
	if next(self.db.char.available) == nil then
		return ""
	else
		return self.db.char.available[math.random(table.getn(self.db.char.available))]
	end
end

function Emotional:IsEnableInParty(info)
	return self.db.char.enableInParty
end

function Emotional:IsEnableInRaid(info)
	return self.db.char.enableInRaid
end

function Emotional:IsEnableInPvP(info)
	return self.db.char.enableInPvP
end

function Emotional:IsEnableInLFR(info)
	return self.db.char.enableInLFR
end

function Emotional:ToggleEnableInParty(info, newValue)
	self.db.char.enableInParty = newValue
end

function Emotional:ToggleEnableInRaid(info, newValue)
	self.db.char.enableInRaid = newValue
end

function Emotional:ToggleEnableInPvP(info, newValue)
	self.db.char.enableInPvP = newValue
end

function Emotional:ToggleEnableInLFR(info, newValue)
	self.db.char.enableInLFR = newValue
end

function Emotional:IsEnableDebug(info)
	return self.db.char.enableDebug
end

function Emotional:ToggleEnableDebug(info, newValue)
	self.db.char.enableDebug = newValue
end